(module talk (lib "slideshow.ss" "slideshow")
  (require "chain.ss"
           (lib "mred.ss" "mred")
           (lib "class.ss")
           (lib "list.ss")
           (lib "code.ss" "slideshow")
           (lib "balloon.ss" "texpict")
           (only (lib "math.ss") pi))

  ;; For the original talk, these were all #f:

  (define show-need-delim? #t)
  (define show-tag-rationale? #t)

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  Title slide

  (define (author who)
    (colorize (bt who) (current-title-color)))
  
  (define (place . where)
    (scale/improve-new-text
     (apply
      hc-append
      (pict-width (t " "))
      (map t where))
     0.75))

  (define plt-logo
    (if printing?
        (bitmap (build-path (collection-path "icons") "PLT-206.png"))
        (cc-superimpose
         (bitmap "plt-shadow.png")
         (bitmap "plt.png"))))
    
  (define (title-slide)
    (slide/center
     (vc-append
      (/ gap-size 2)
      (titlet "Adding Delimited and Composable Control")
      (titlet "to a Production Programming Environment"))
     (blank)
     plt-logo
     (blank)
     (table 2
            (list
             (author "Matthew Flatt")
             (place "University of Utah")
             (author "Gang Yu")
             (place "Institute of Software,"
                    "Chinese Academy of Sciences")
             (author "Robert Bruce Findler")
             (place "University of Chicago")
             (author "Matthias Felleisen")
             (place "Northeastern University"))
            (cons rbl-superimpose lbl-superimpose) lbl-superimpose
            gap-size (* 4 line-sep))))
  
  ;; ----------------------------------------

  (define big-code-scale 1.5)

  (slide/center
   (scale/improve-new-text
    (code (call/cc (lambda (k) 
                     (set! start-k k))))
    big-code-scale))
  
  (title-slide)
  
  (define (quiz what does answer show-answer? flatten)
    (vc-append
     gap-size
     (ht-append
      gap-size
      (scale/improve-new-text
       (cc-superimpose (colorize (bt "?") "purple")
                       (scale (colorize (t "?") "lightblue") 0.95))
       6)
      (inset
       (vl-append 50
                  (t "Name")
                  (let ([p (inset (scale (bt what) big-code-scale) 50 0 0 0)])
                    (cc-superimpose
                     (ghost p)
                     (if (zero? flatten)
                         (blank)
                         (scale p 1 flatten))))
                  (t "with")
                  (inset (hbl-append
                          (scale (t "first-class continuations") 
                                 big-code-scale)
                          (t "..."))
                         100 0 0 0))
       0 50 0 0))
     (blank)
     (blank)
     ((if show-answer? values ghost)
      (colorize (scale (bt answer)
                       (* big-code-scale big-code-scale))
                "forestgreen"))))
  
  (define (quiz1 show-answer? flatten)
    (quiz "a programming language" "provides" "Scheme" show-answer? flatten))
  
  (slide/center (quiz1 #f 1))

  (define (show-restart)
    (slide/center
     (scale/improve-new-text
      (code (start-k))
      big-code-scale))
    (title-slide))
  
  (show-restart)

  (slide/center (quiz1 #f 1))
  
  (define (quiz2 show? flatten)
    (quiz "an application" "uses" "web servers" show? flatten))
  
  (play #f (lambda (n)
             (if (zero? n)
                 (quiz1 #t 1)
                 (if (n . < . 0.5)
                     (quiz1 #f (* 2 (- 0.5 n)))
                     (quiz2 #f (* 2 (- n 0.5)))))))
  
  (slide/center (quiz2 #f 1))
  (slide/center (quiz2 #t 1))

  ;; ----------------------------------------

  (define (servlet-slide balloon?)
    (slide/title/center
     "Web Servlet with Continuations"
     (let ([p (code
               (define (paper-search-servlet)
                 #,(get-prog pick-flight-expr)))]) 
       (if balloon?
           (pin-balloon
            (wrap-balloon (vl-append
                           (* 3 line-sep)
                           (t "send back HTML form, then")
                           (t "wait for answer as new request"))
                          's 0 (* 2 gap-size)
                          my-balloon-color)
            p
            pick-flight-expr ct-find)
           p))))
  (servlet-slide #f)
  (servlet-slide #t)
  
  (play-n "Implementing a Web Server" web-server-mid)

  (define web-server/prompt-title "Web Server with Prompt")
  
  (play-n web-server/prompt-title (web-server-delim-mid #f))
  
  (slide/title/center
   "Rolling Your Own vs. Language Extension"
   (page-para "Delimited control via" p:call/cc "doesn’t work right with")
   (npage-item "exceptions")
   (npage-item "dynamic binding")
   (npage-item p:dynamic-wind)
   'next
   (blank)
   (blank)
   (page-para* (bt "Our goal:")
               "delimited control integrated with existing constructs"))
   

  ;; ----------------------------------------

  (define cite-color (make-object color% 50 50 50))
  
  (define (cite who when)
    (colorize (hbl-append (t who)  (t " ") (t when))
              cite-color))
  
  (define design-papers
    (list (list "Felleisen" "88")
          (list "Danvy and Filinski" "90")
          (list "Hieb and Dybvig" "90")
          (list "Sitaram and Felleisen" "90")
          (list "Queinnec and Serpette" "91")
          (list "Sitaram" "93")
          (list "Moreau and Queinnec" "94")
          (list "Gunter et al." "95")
          (list "Kiselyov et al." "06")
          (list "Dybvig et al." "06")))
  
  (define more-papers
    (list (list "Queinnec" "93")
          (list "Wadler" "94")
          (list "deGroote" "94")
          (list "Rehof and Sørensen" "94")
          (list "Gunter et al." "95")
          (list "Thielecke" "97")
          (list "Rehof" "01")
          (list "Gasbichler and Sperber" "02")
          (list "Kameyama and Hasegawa" "03")
          (list "Ariola et al." "04")
          (list "Shan" "04")
          (list "Saurin" "05")
          (list "Biernacki et al." "06")))
  
  (define (evenize-l l)
    (if (even? (length l))
        l
        (append l (list (blank)))))
  
  (slide/title/center
   "Papers on Delimited Continuations"
   (table 2
          (evenize-l
           (map (lambda (p) (cite (car p) (cadr p)))
                (sort (append
                       design-papers
                       more-papers)
                      (let ([expand (lambda (x)
                                      (if (char=? #\0 (string-ref x 0))
                                          (string-append "20" x)
                                          (string-append "19" x)))])
                        (lambda (a b)
                          (string<? (expand (cadr a)) (expand (cadr b))))))))
          ltl-superimpose ltl-superimpose
          (* 2 gap-size) (* 3 line-sep)))

  (slide/title/center
   "Papers on Design"
   (apply vl-append (* 3 line-sep)
          (map (lambda (p) (cite (car p) (cadr p)))
               design-papers)))

  (slide/title/center
   "Papers on Implementation"
   (vl-append (* 3 line-sep)
              (cite "Gasbichler and Sperber" "02")              
              (cite "Dybvig et al." "06")))

  (slide/title/center
   "Implementations")

  (slide/title/center
   "Implementations (Now)"
   plt-logo)
  
  (slide/title/center
   "Balance"
   (let* ([p (scale (bitmap "table.jpg") 0.8)]
          [rt (lambda (s r)
                (let ([w (let ([d (blank)])
                           (refocus (cc-superimpose
                                     d
                                     (text s (current-main-font) (current-font-size) r))
                                    d))])
                  (lt-superimpose (inset (colorize w "white") 1 1)
                                  (inset (colorize w "white") -1 -1)
                                  w)))]
          [gp (ghost p)])
     (pin-under (vc-append
                 (ht-append
                  (rt "exceptions" (* pi 0.2))
                  (blank (* 0.95 (pict-width p)) 0)
                  (rt "dynamic binding" (* pi -0.2)))
                 (hc-append (* 2 gap-size)
                            (rt "prompts" (* pi 0.5))
                            gp
                            (rt "abort" (* pi -0.5)))
                 (ht-append
                  (rt "dynamic-wind" (* pi 0.8))
                  (blank (* 0.95 (pict-width p)) 0)
                  (rt "continuations" (* pi -0.8))))
                gp lt-find
                p)))
  
  (define (contrib-slide design-model-link)
    (slide/title/center
     "Contributions"
     (vl-append
      (* 4 gap-size)
      (cc-superimpose
       (vl-append 
        (* 4 gap-size)
        (page-item* "Comprehensive design")
        (page-item* "Formal model"))
       design-model-link)
      (page-item* "Implementation"))))
  
  (contrib-slide (blank))
  (contrib-slide (colorize
                  (vc-append
                   (arrow gap-size (/ pi 2))
                   (it "graphical intuition")
                   (arrow gap-size (/ pi -2)))
                  "blue"))
  
  (show-restart)

  ;; ----------------------------------------

  (define notation-title "Notation")
  
  (map (lambda (mode)
         (play notation-title (orig-defn-steps-mid mode)))
       '(hilite-inner copy-inner
                      hilite-middle copy-middle
                      hilite-outer copy-outer
                      finish))

  (play notation-title chain1-defn-mid)
  
  (define reduction-title "Reductions")

  (play reduction-title chain1-defn-leads-mid)
  (play reduction-title chain1-mid)
  (play reduction-title chain1-un-mid)  
  (play reduction-title chain1-to-chain2-mid)
  
  (define cont-title "Continuations")
  
  (play cont-title chain2-capture-mid)
  (play cont-title chain2-shift-mid)
  (play cont-title chain2-leave-mid)
  (play cont-title chain2-restore-mid)
  (play cont-title chain2-reduce-mid)

  (play cont-title (reverse-animations chain2-leave-mid
                                       chain2-restore-mid
                                       chain2-reduce-mid))
  
  (play cont-title chain2-to-comp-mid)
  
  (define comp-title "Composable Continuations")
  
  (play comp-title chain2-comp-app-mid)
  (play comp-title chain2-comp-finish-mid)
  
  (when show-need-delim?
    (play comp-title chain2-comp-add-mid)
    (play comp-title chain2-comp-addadd-mid))
  
  (play comp-title (sweep ((if show-need-delim?
                               chain2-comp-addadd-mid
                               chain2-comp-finish-mid)
                           1.0) 
                          (chain3-copy-mid 0.0)))

  ;; prompts:
  (define delim-capture-title "Delimited Capture")
  (define delim-abort-title "Delimited Abort")

  (play delim-capture-title chain3-copy-mid)
  (play delim-capture-title chain3-app-mid)

  (play delim-capture-title (sweep (chain3-app-mid 1.0)
                                   (chain3-drop-mid 0.0)))
  
  (play delim-abort-title chain3-drop-mid)
  (play delim-abort-title chain3-jump-mid)
  
  (slide/title/center
   delim-abort-title
   (chain3-jump-mid 1.0))
  
  (define (split-slide proc balloon?)
    (slide/title/center
     "Splitting Capture and Abort"
     (npage-item p:call/comp " : capture current continuation")
     (npage-item p:abort " : abort current continuation")
     (blank)
     (npage-item F-op " : capture and abort current continuation")
     (blank)
     (proc
      (let* ([thunk (code λ)]
             [proc (htl-append F-op (tt " = ")
                               (code (λ (f) (call/comp 
                                             (λ (k) (abort (#,thunk () (f k))))))))])
        (if balloon?
            (pin-balloon
             (wrap-balloon (para*/c (/ client-w 4)
                                    "delay" (code (f k)) "until after abort")
                           'n
                           0 (- gap-size)
                           my-balloon-color)
             proc
             thunk
             cb-find)
            proc)))))

  (split-slide ghost #f)
  (split-slide values #f)
  (split-slide values #t)
  
  ;; tagged prompts:
  (define handler-title "Prompt with Handler")
  (define catch-title "Catch and Throw")
  (define tagged-title "Tagged Prompts")
  (when show-tag-rationale?
    (play handler-title chain6-drop-mid)
    (play handler-title chain6-app-mid)
    (play handler-title chain6-F-mid)
    (play handler-title (reverse-animations chain6-drop-mid
                                            chain6-app-mid
                                            chain6-F-mid))
    (play handler-title chain6-to-catch-mid)
    (play catch-title chain6-insert-mid)
    (play catch-title chain6-amb-mid)
    (play catch-title chain6-tagged-mid)
    (play catch-title chain6-hilite-mid)
    
    (play tagged-title (sweep (chain6-hilite-mid 1.0)
                              (chain5-comp-hilite-mid 0.0)))
    
    (play tagged-title chain5-comp-hilite-mid)
    (play tagged-title chain5-copy-mid)
    (play tagged-title chain5-app-mid)
    (play tagged-title chain5-abort-hilite-mid)
    (play tagged-title chain5-drop-mid)
    (play tagged-title chain5-abort-mid)
    (slide/title/center tagged-title (chain5-abort-mid 1.0)))
  
  (slide/title/center
   "Continuations Summary"
   (npage-item p:prompt "with handler and tag")
   (npage-item p:abort "with tag")
   (npage-item p:call/comp "with tag")
   (npage-item "continuation composition")
   (blank)
   (npage-item "plain" p:call/cc "uses default tag")
   (blank)
   (blank)
   (colorize (page-para/r "[Sitaram PLDI'93]") "blue"))
  
  ;; ----------------------------------------------------------------------
  
  ;; wcm:
  (define mark-title "Continuation Marks")

  #;
  (begin
    (play mark-title chain8-mark1-mid)
    (play mark-title chain8-step1-mid)
    (play mark-title chain8-mark2-mid)
    (play mark-title chain8-comp-mid)
    (play mark-title chain8-copy-mid))
  
  (slide/title/center mark-title (chain8-copy-mid 1.0))
  
  #;
  (begin
    (slide/title/center 
     "Getting Marks" 
     chain7-eval)
    (slide/title/center
     "Capturing Marks" 
     chain9-eval)
    (slide/title/center
     "Restoring Marks"
     (t "(need a simple, non-merging example here)"))
    (slide/title/center 
     "Getting Delimited Marks"
     chain11-eval)
    (slide/title/center 
     "Merging Marks" 
     chain10-eval))

  (slide/title/center
   "Dynamic Binding Summary"
   (npage-item p:call/cm "to add marks")
   (npage-item p:current-marks "to get marks, up to a tag")
   (blank)
   (npage-item "capture marks in" p:call/comp)
   (npage-item "splice marks in continuation composition"))

  ;; ----------------------------------------------------------------------

  ;; dw
  (define dw-title "Dynamic Wind")
  (play "Side Effects and Control" dw-code-begin-mid)
  (play dw-title dw-code-example-mid)
  (play dw-title chainD7-example-to-generic-mid)
  (play dw-title chainD7-generic-dw-mid)
  (play dw-title chainD7-generic-redex-mid)
  (play dw-title chainD7-generic-spurs-mid)
  (play dw-title (reverse-animations chainD7-generic-redex-mid
                                     chainD7-generic-spurs-mid))
  
  (define dw-eval-title "Dynamic Wind Evaluation")
  
  (play dw-eval-title chainD7-generic-predex-mid)
  (play dw-eval-title chainD7-generic-prebegin-mid)
  (play dw-eval-title chainD7-generic-prereduces-mid)
  (play dw-eval-title chainD7-generic-predone-mid)
  (play dw-eval-title chainD7-generic-redex-mid)
  (play dw-eval-title chainD7-generic-spurs-mid)
  
  (define dw-jump-title "Dynamic Wind and Jumps")
  
  (play dw-jump-title chainD7-generic-to-abort-mid)
  (play dw-jump-title chainD7-exit-mid)
  (play dw-jump-title chainD7-add-jump-in-mid)
  (play dw-jump-title chainD7-jump-in-mid)
  (play dw-jump-title chainD7-remove-jump-in-mid)

  (define dw-abort-title "Dynamic Wind and Abort Evaluation")

  (play dw-abort-title chainD7-start-abort-mid)
  (play dw-abort-title chainD7-abort-save-mid)
  (play dw-abort-title chainD7-abort-post-mid)

  (play dw-abort-title
        (sweep (chainD7-abort-post-mid 1.0)
               (dw-share-copy-mid 0.0)))

  (define dw-cont-title "Dynamic Wind and Call/cc")

  (play dw-cont-title dw-share-copy-mid)
  (play dw-cont-title dw-share-eval1-mid)
  (play dw-cont-title dw-share-eval2-mid)
  (play dw-cont-title dw-share-eval3-mid)
  (slide/title/center
   dw-cont-title
   (dw-share-eval3-mid 1.0))
  (play dw-cont-title dw-share-think-mid)
  (play dw-cont-title dw-share-orig-mid)
  (play dw-cont-title dw-share-share-mid)
  (play dw-cont-title dw-share-revised-mid)
  (play dw-cont-title dw-share-tagged-mid)
  (play dw-cont-title dw-share-abort-drop-mid)
  (play dw-cont-title dw-share-abort-mid)
  (play dw-cont-title dw-share-post-mid)
  (play dw-cont-title dw-share-postval-mid)
  (play dw-cont-title dw-share-done-mid)
  (slide/title/center
   dw-cont-title
   (dw-share-done-mid 1.0))

  (slide/title/center
   "Dynamic Wind Summary"
   (npage-item p:dynamic-wind "generates" p:dw)   
   (npage-item p:call/cc "detects sharing in continuation jumps")
   (blank)
   (npage-item "capture" p:dw "thunks in" p:call/comp "and" p:call/cc)
   (npage-item "run post thunks in" p:abort)
   (npage-item "run pre thunks in continuation composition"))
  
  ;; ----------------------------------------

  (slide/title
   "To Continue..."
   (lpage-item "Read the paper")
   (blank)
   (blank)
   (lpage-item "Run the Redex model:")
   (tt "http://www.cs.utah.edu/plt/delim-cont/")
   (blank)
   (blank)
   (lpage-item "Download the implementation:")
   (tt "http://www.plt-scheme.org/")
   'next
   (blank)
   (blank)
   (scale/improve-new-text
    (code (web-k #,search-terms-expr))
    big-code-scale))
  
  (slide/title/center
   web-server/prompt-title
   (let* ([f (web-server-delim-mid #t)]
          [n (procedure-arity f)])
     (apply f (vector->list (make-vector n 1.0)))))

  
  (void))
